// 使用するハードウェア　Espressif System ESP32(ESP-WROOM-32)
#include <WiFi.h>
// 通信プロトコルにUDPを採用
#include "AsyncUDP.h" // UDP

#define settingPin 32  // 接地時はIPドレス等設定（シリアル端末にて）
#define wifiConnectedPin 23  // wifi接続中表示
#define encoderApin 19  // EncoderA
#define encoderBpin 21  // EncoderB

// ここから メイン.ino と net-param-SPIFFS.ino との連携データ
bool settingMode = true;
String sketch;        // メインスケッチファイル名など
int debug;            // 0:本番
String myplace;       // このプログラムが動作する場所（server / client）
char myssid[31];      // Wi-Fiルーターのssid （charの場合は文字数指定が必要）
char mypassword[31];  // Wi-Fiルーターのパスワード
String myportStr;     // myport番号は指定しても無効（制限事項）
uint16_t  myport;     // myport番号は指定しても無効（制限事項）
IPAddress myip(192,168, 0, 49);      //myipaddress　仮の値
IPAddress mygateway(192,168, 0, 1);  //mygateway　　　〃
IPAddress mysubnet(255, 255,255, 0); //mysubnetmask　〃
IPAddress mydns(192,168, 0, 1);      //mydns　       〃
String hisNetname; //  hisipのネットアドレス型式名
IPAddress hisip(49,49,49,49); //動作場所が"client"の場合に、his即ちサーバーの指定に使用
unsigned int hisport;         //動作場所が"client"の場合に、his即ちサーバーの指定に使用
String appPW;                 //アプリレベルの認証パスワード
// ここまで連携データ

// ESP32でUDP通信を行う
//  https://program4ptotat.livedoor.blog/archives/11904553.html
AsyncUDP udp; // udpは勝手につけた名前なので何でも大丈夫です。

int stas = 0;  // CONNECT時の状態遷移
volatile int rotaEncodeCount =0;

//  参考Web　Marchan  ESP32でのタイマー割り込みによる自動計測 
// http://marchan.e5.valueserver.jp/cabin/comp/jbox/arc202/doc21105.html
volatile int timer2Ctr = 0; // 割込通知カウンター
hw_timer_t *timer2 = NULL; // タイマー設定用ポインター
portMUX_TYPE timerMux = portMUX_INITIALIZER_UNLOCKED; // 同期管理用変数

void IRAM_ATTR onTimer2(){ // LED点滅 ★結局割込みを使わない点滅方法を採用
  portENTER_CRITICAL_ISR(&timerMux); // クリティカルセクション
  timer2Ctr ++;
  portEXIT_CRITICAL_ISR(&timerMux); // クリティカルセクション
}

//特にWi-Fi接続に参考にしたWebサイト
//https://github.com/espressif/arduino-esp32/blob/master/libraries/WiFi/examples/SimpleWiFiServer/SimpleWiFiServer.ino
//mgo-tec電子工作
//    https://www.mgo-tec.com/blog-entry-esp-wroom-32-esp32-wifi-smartphone-led-switch.html/4
//WiFiServer server(60049); // PORT No. ***** ポート番号の可変化が未プログラミング
WiFiClient client;// ***** loop　へ移動させよう 

// 192.168.0.1　形式でアドレスをプリントする
void PrintIPaddress(IPAddress ip){
  Serial.print(ip[0], DEC);Serial.print('.');Serial.print(ip[1], DEC);Serial.print('.');
  Serial.print(ip[2], DEC);Serial.print('.');Serial.print(ip[3], DEC);
}

void setup() {
  int i;    //temporary work
  int j;    //temporary work
  String strTemp;
  debug = 0; // debug mode

  delay(500);
  Serial.begin(115200); // シリアルモニターをこの速度でオープン
  pinMode(settingPin, INPUT_PULLUP);
  pinMode(wifiConnectedPin, OUTPUT);
  digitalWrite(wifiConnectedPin, LOW);
  pinMode(encoderApin, INPUT_PULLUP); // プルアップ
  pinMode(encoderBpin, INPUT_PULLUP); // プルアップ
  delay(10);
 
  // ネットワークパラメターの設定
  if(digitalRead(settingPin)) { // run mode
    settingMode = false;
  }
  else{ // setting mode
    settingMode = true;
  }
  //run mode  ファイルが無ければ既定値でファイルを作成し、既定値でリモートキー動作開始。
  //          ファイルが有ればファイル内容でリモートキー動作開始。
  //setting mode  設定コマンド処理を実行。ファイルが有ればその内容で、設定コマンド処理を実行。
  //              exitコマンドで終了（return）。
  setNetworkParameter();  // 別inoファイル(network_parameter_setting_function.ino)に有り
  
  //TIMER2の設定 //  LED点滅タイマー（エンコーダーを回したことの知らせ）
  timer2 = timerBegin(2, 80, true);  //プリスケーラを80分周＝1マイクロ秒ごとにインクリメント　(ESP Reference参照)
  timerAttachInterrupt(timer2, &onTimer2, true);  //タイムアウトした時に行う処理(関数)を定義
  timerAlarmWrite(timer2, 100000, true);  // タイマー割込み間隔 0.1sec  
  // timerAlarmEnable(timer2);  //タイマー２スタート　 ★結局割込みを使わない点滅方法を採用
}

//  マルチタスク
//  参考Web　mgo-tec電子工作  ESP32のマルチタスク( Dual Core)を試す 
//  https://www.mgo-tec.com/blog-entry-arduino-esp32-multi-task-dual-core-01.html/3
TaskHandle_t th[2];  //  重要
// RotalyEncoderからのデータ処理
void Task1(void *pvParameters){ // メインLoopと同じCPU上の別タスク
  unsigned char Apin;
  unsigned char Bpin;
  unsigned char rotaEnc_now;  // value of rotary encoder
  unsigned char rotaEnc_old;  // value of rotary encoder 
  signed char rotaEncodeValue; // -1,0,1
  //　-1が左回転　0は回転なし　1が右回転  参照の添え字は0から数える
  signed char rotaEnc_stats[] = { 0, 1,-1, 0,-1, 0, 0, 1, 1, 0, 0,-1, 0,-1, 1, 0};
  while(1){
              Apin = digitalRead(encoderApin);
              Bpin = digitalRead(encoderBpin);
              // 回転方向の検出原理：次のWebsiteの説明が分かりやすい
              // http://tylercsf.blog123.fc2.com/blog-category-38.html
              //             　   A           B　　　　   　　    　　回転方向の検出ロジック
              rotaEnc_now = (Apin << 1) | Bpin;                   // Aを1ビットシフト | B　 ビット演算のOR
              //rotaEnc_now = (encoderApin << 1) | encoderBpin;   // Aを1ビットシフト | B　 ビット演算のOR
              rotaEnc_old = rotaEnc_old << 2;                     // 2ビットシフト
              rotaEnc_old = rotaEnc_old | (rotaEnc_now & 0x03);   // old | now 　 ビット演算のOR
              rotaEncodeValue = rotaEnc_stats[rotaEnc_old & 0x0F];           //　下位4ビットを生かし配列から値を取得
        
              if(rotaEncodeValue != 0){  //　-1が左回転　0は回転なし　1が右回転
                rotaEncodeCount = rotaEncodeCount + rotaEncodeValue;
                // String str = String(rotaEncodeValue);
                // Serial.print(str); // debug
                // udp.print(str);
              }
              // delay(1); // ms Wait エンコーダのチャタリング回避（減少
              delay(10); // ms Wait エンコーダのチャタリング回避（減少）
        delay(1); 
  } // 無限ループ
}

void loop() {
  String recvCmd ;
  char ch;         //CMDの1文字毎受信用
  int i; //temporary work
  int j; //temporary work
  bool wifiConnectedPinLchika = false;
  
  // loopの中なので、WiFi接続が切れた場合の自動再接続が可能
  while (WiFi.status() != WL_CONNECTED) { // loop WiFi not connected
    stas = 0;
    // ＬED点滅
    if(wifiConnectedPinLchika == true){
      digitalWrite(wifiConnectedPin, HIGH); wifiConnectedPinLchika = false;
    }
    else{
      digitalWrite(wifiConnectedPin, LOW); wifiConnectedPinLchika = true;      
    }
    WiFi.begin(myssid, mypassword); // loop内で呼び出す http://skomo.o.oo7.jp/f49/hp49_14.htm
    if(debug != 0) Serial.print("."); // debug
    delay(1000);
  } // loop WiFi not connected
  // WiFi connected
  if(stas == 0){ // ＷiＦi接続直後一回のみ    
    Serial.println("");
    Serial.println("WiFi connected");
    digitalWrite(wifiConnectedPin, HIGH);
    // print your WiFi shield's IP address:
    Serial.print("IP address: ");
    Serial.print(WiFi.localIP());
    // print the received signal strength:
    long rssi = WiFi.RSSI();
    Serial.print("  signal strength:");
    Serial.print(rssi);
    Serial.println(" dBm");

    // ESP32でUDP通信を行う
    //   https://program4ptotat.livedoor.blog/archives/11904553.html
    // ESP32/ArduinoのUDP通信メモ (AsyncUDP)
    //   https://qiita.com/yomori/items/3e02bec35ad1262b7b94
    if(udp.connect(hisip, 55600)) { // ServerAddress, portNo **** ここで指定する
      // まちがったIPアドレスを指定しても　true　になる?????
      Serial.println("UDP connected");
    }
    else{
      delay(5000);      
    }
       
    //  マルチタスク
    //  参考Web　mgo-tec電子工作  ESP32のマルチタスク( Dual Core)を試す 
    //  https://www.mgo-tec.com/blog-entry-arduino-esp32-multi-task-dual-core-01.html/3
    //  (Core0 で Wi-Fi や Bluetooth などの制御用のプログラムが動作。) 
    //  (Core1 でメインloopが動作。) 
    //  Task1(別タスク)の起動。Wi-Fiなどとの競合さけるためにCore1で別タスクで動作させる。
    //  xTaskCreatePinnedToCore(タスク名, "タスク名" ,スタックメモリサイズ, NULL, 
    //                          タスク優先順位, タスクハンドルポインタ, Core ID);
    xTaskCreatePinnedToCore(Task1, "Task1", 4096, NULL, 1, &th[0], 1);     
    stas = 1;
  } // ＷiＦi接続直後一回のみ

  if(rotaEncodeCount != 0) {
    digitalWrite(wifiConnectedPin, LOW); // ＬED点滅
    String str = String(rotaEncodeCount);
    if(debug != 0) Serial.print(str); // debug
    udp.print(str); // ＵＤＰ送信
    rotaEncodeCount = 0;
 }
  
  delay(100); // ***** 要調整　短すぎると速く回した場合にクリック数が追いつかない
  digitalWrite(wifiConnectedPin, HIGH); // ＬED点滅
  // delay(10000); // debug
}
